local famBags = LibStub('AceAddon-3.0'):GetAddon('famBags')
famBags.Classification = {}

-- :::::::::::: LOCALS ::::::::::::: -------------------------------------------------------------------------

-- :::::::::::: famBags.Classification ::::::::::::: -------------------------------------------------------------------------
function famBags.Classification:GetButtonCategory(info)
   local db = famBags.db.profile.categories
   local cat, result, val
   for _, catname in pairs(famBags.SortedCategories) do
      cat = db[catname]
      result = info.id and self:Types(cat, info)
      if cat.Criterias then
         for i, crit in ipairs(cat.Criterias) do
            val = self[crit.name](crit, info)
            if result == nil then 
               result = val
            elseif crit.operator == 'AND' then
               result = result and val
            elseif crit.operator == 'AND NOT' then
               result = result and not val
            elseif crit.operator == 'OR' then
               result = result or val
            elseif crit.operator == 'OR NOT' then
               result = result or not val
            end
         end
      end
      if result then return catname end
   end
   return nil
end

function famBags.Classification:Types(cat, info)
   if cat.Types and next(cat.Types) then
      if not info.type then return false end
      return (rawget(cat.Types, info.type) and 
             (rawget(cat.Types[info.type], famBags.L['.: All :.']) or 
              rawget(cat.Types[info.type], info.subType))) or false
   else
      return nil
   end
   return false
end

famBags.Classification['Others'] = function(crit, info)
   return true
end

famBags.Classification['Empty'] = function(crit, info)
   return (not info.id) or false
end

famBags.Classification['Quest'] = function(crit, info)
   return info.isQuestItem or false
end
              
famBags.Classification['Usable'] = function(crit, info)
   return info.spell or false
end

famBags.Classification['Key'] = function(crit, info)
   return info.bag == KEYRING_CONTAINER
end

famBags.Classification['Quality'] = function(crit, info)
   if not info.quality then return false end
   local quality = (info.quality >= 0) and info.quality..'.'.._G[string.format('ITEM_QUALITY%s_DESC', info.quality)] or famBags.L['unknown']
   return (rawget(crit.criterias, famBags.L['.: All :.']) or rawget(crit.criterias, quality)) or false
end

famBags.Classification['Equipment Set'] = function(crit, info)
   for _, items in pairs(famBags.EquipmentSets) do
      if items and rawget(items, info.id) then
         return true
      end      
   end 
   return false
end

